@@ -3,7 +3,7 @@ Tools for drawing Python object reference graphs with graphviz.
 
 You can find documentation online at http://mg.pov.lt/objgraph/
 
-Copyright (c) 2008-2010 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2008-2014 Marius Gedminas <marius@pov.lt>
 Copyright (c) 2010 Stefano Rivera <stefano@rivera.za.net>
 
 Released under the MIT licence.
@@ -27,10 +27,10 @@ Released under the MIT licence.
 # DEALINGS IN THE SOFTWARE.
 
 __author__ = "Marius Gedminas (marius@gedmin.as)"
-__copyright__ = "Copyright (c) 2008-2012 Marius Gedminas"
+__copyright__ = "Copyright (c) 2008-2014 Marius Gedminas"
 __license__ = "MIT"
-__version__ = "1.7.2"
-__date__ = "2012-10-23"
+__version__ = "1.8.0"
+__date__ = "2014-02-13"
 
 
 import codecs
@@ -68,25 +68,34 @@ def count(typename, objects=None):
         42
         >>> count('MyClass', get_leaking_objects())
         3
+        >>> count('mymodule.MyClass')
+        2
 
     Note that the GC does not track simple objects like int or str.
 
     .. versionchanged:: 1.7
        New parameter: ``objects``.
 
+    .. versionchanged:: 1.8
+       Accepts fully-qualified type names (i.e. 'package.module.ClassName')
+       as well as short type names (i.e. 'ClassName').
+
     """
     if objects is None:
         objects = gc.get_objects()
-    return sum(1 for o in objects if type(o).__name__ == typename)
+    if '.' in typename:
+        return sum(1 for o in objects if long_typename(o) == typename)
+    else:
+        return sum(1 for o in objects if type(o).__name__ == typename)
 
 
-def typestats(objects=None):
+def typestats(objects=None, shortnames=True):
     """Count the number of instances for each type tracked by the GC.
 
     Note that the GC does not track simple objects like int or str.
 
     Note that classes with the same name but defined in different modules
-    will be lumped together.
+    will be lumped together if ``shortnames`` is True.
 
     Example:
 
@@ -100,17 +109,24 @@ def typestats(objects=None):
     .. versionchanged:: 1.7
        New parameter: ``objects``.
 
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
+
     """
     if objects is None:
         objects = gc.get_objects()
+    if shortnames:
+        typename = short_typename
+    else:
+        typename = long_typename
     stats = {}
     for o in objects:
-        stats.setdefault(type(o).__name__, 0)
-        stats[type(o).__name__] += 1
+        n = typename(o)
+        stats[n] = stats.get(n, 0) + 1
     return stats
 
 
-def most_common_types(limit=10, objects=None):
+def most_common_types(limit=10, objects=None, shortnames=True):
     """Count the names of types with the most instances.
 
     Returns a list of (type_name, count), sorted most-frequent-first.
@@ -130,15 +146,18 @@ def most_common_types(limit=10, objects=None):
     .. versionchanged:: 1.7
        New parameter: ``objects``.
 
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
+
     """
-    stats = sorted(typestats(objects).items(), key=operator.itemgetter(1),
-                   reverse=True)
+    stats = sorted(typestats(objects, shortnames=shortnames).items(),
+                   key=operator.itemgetter(1), reverse=True)
     if limit:
         stats = stats[:limit]
     return stats
 
 
-def show_most_common_types(limit=10, objects=None):
+def show_most_common_types(limit=10, objects=None, shortnames=True):
     """Print the table of types of most common instances.
 
     The caveats documented in :func:`typestats` apply.
@@ -157,14 +176,17 @@ def show_most_common_types(limit=10, objects=None):
     .. versionchanged:: 1.7
        New parameter: ``objects``.
 
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
+
     """
-    stats = most_common_types(limit, objects)
+    stats = most_common_types(limit, objects, shortnames=shortnames)
     width = max(len(name) for name, count in stats)
     for name, count in stats:
         print('%-*s %i' % (width, name, count))
 
 
-def show_growth(limit=10, peak_stats={}):
+def show_growth(limit=10, peak_stats={}, shortnames=True):
     """Show the increase in peak object counts since last call.
 
     Limits the output to ``limit`` largest deltas.  You may set ``limit`` to
@@ -185,9 +207,13 @@ def show_growth(limit=10, peak_stats={}):
         ...
 
     .. versionadded:: 1.5
+
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
+
     """
     gc.collect()
-    stats = typestats()
+    stats = typestats(shortnames=shortnames)
     deltas = {}
     for name, count in iteritems(stats):
         old_count = peak_stats.get(name, 0)
@@ -224,7 +250,7 @@ def get_leaking_objects(objects=None):
         # this then is our set of objects without referrers
         return [i for i in objects if id(i) in ids]
     finally:
-        objects = i = j = None # clear cyclic references to frame
+        objects = i = None # clear cyclic references to frame
 
 
 def by_type(typename, objects=None):
@@ -240,10 +266,17 @@ def by_type(typename, objects=None):
     .. versionchanged:: 1.7
        New parameter: ``objects``.
 
+    .. versionchanged:: 1.8
+       Accepts fully-qualified type names (i.e. 'package.module.ClassName')
+       as well as short type names (i.e. 'ClassName').
+
     """
     if objects is None:
         objects = gc.get_objects()
-    return [o for o in objects if type(o).__name__ == typename]
+    if '.' in typename:
+        return [o for o in objects if long_typename(o) == typename]
+    else:
+        return [o for o in objects if type(o).__name__ == typename]
 
 
 def at(addr):
@@ -302,7 +335,7 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
 
     Example:
 
-        >>> find_backref_chain(obj, inspect.ismodule)
+        >>> find_backref_chain(obj, is_proper_module)
         [<module ...>, ..., obj]
 
     Returns ``[obj]`` if such a chain could not be found.
@@ -317,7 +350,7 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
 
 def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
                   highlight=None, filename=None, extra_info=None,
-                  refcounts=False):
+                  refcounts=False, shortnames=True):
     """Generate an object reference graph ending at ``objs``.
 
     The graph will show you what objects refer to ``objs``, directly and
@@ -350,6 +383,10 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     These will mostly match the number of arrows pointing to an object,
     but can be different for various reasons.
 
+    Specify ``shortnames=False`` if you want to see fully-qualified type
+    names ('package.module.ClassName').  By default you get to see only the
+    class name part.
+
     Examples:
 
         >>> show_backrefs(obj)
@@ -365,16 +402,20 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     .. versionchanged:: 1.5
        New parameter: ``refcounts``.
 
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
+
     """
     show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
                filter=filter, too_many=too_many, highlight=highlight,
                edge_func=gc.get_referrers, swap_source_target=False,
-               filename=filename, extra_info=extra_info, refcounts=refcounts)
+               filename=filename, extra_info=extra_info, refcounts=refcounts,
+               shortnames=shortnames)
 
 
 def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
               highlight=None, filename=None, extra_info=None,
-              refcounts=False):
+              refcounts=False, shortnames=True):
     """Generate an object reference graph starting at ``objs``.
 
     The graph will show you what objects are reachable from ``objs``, directly
@@ -420,14 +461,18 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
        New parameters: ``filename``, ``extra_info``.
 
     .. versionchanged:: 1.5
-       New parameter: ``refcounts``.
        Follows references from module objects instead of stopping.
+       New parameter: ``refcounts``.
+
+    .. versionchanged:: 1.8
+       New parameter: ``shortnames``.
 
     """
     show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
                filter=filter, too_many=too_many, highlight=highlight,
                edge_func=gc.get_referents, swap_source_target=True,
-               filename=filename, extra_info=extra_info, refcounts=refcounts)
+               filename=filename, extra_info=extra_info, refcounts=refcounts,
+               shortnames=shortnames)
 
 
 def show_chain(*chains, **kw):
@@ -436,20 +481,21 @@ def show_chain(*chains, **kw):
     Useful in combination with :func:`find_ref_chain` or
     :func:`find_backref_chain`, e.g.
 
-        >>> show_chain(find_backref_chain(obj, inspect.ismodule))
+        >>> show_chain(find_backref_chain(obj, is_proper_module))
 
     You can specify if you want that chain traced backwards or forwards
     by passing a ``backrefs`` keyword argument, e.g.
 
-        >>> show_chain(find_ref_chain(obj, inspect.ismodule),
+        >>> show_chain(find_ref_chain(obj, is_proper_module),
         ...            backrefs=False)
 
     Ideally this shouldn't matter, but for some objects
     :func:`gc.get_referrers` and :func:`gc.get_referents` are not perfectly
     symmetrical.
 
-    You can specify ``highlight``, ``extra_info`` or ``filename`` arguments
-    like for :func:`show_backrefs` or :func:`show_refs`.
+    You can specify ``highlight``, ``extra_info``, ``refcounts``,
+    ``shortnames`` or ``filename`` arguments like for :func:`show_backrefs` or
+    :func:`show_refs`.
 
     .. versionadded:: 1.5
 
@@ -469,6 +515,27 @@ def show_chain(*chains, **kw):
         show_refs([chain[0] for chain in chains], max_depth=max_depth,
                   filter=in_chains, **kw)
 
+
+def is_proper_module(obj):
+    """
+    Returns ``True`` if ``obj`` can be treated like a garbage collector root.
+
+    That is, if ``obj`` is a module that is in ``sys.modules``.
+
+    >>> import imp
+    >>> is_proper_module([])
+    False
+    >>> is_proper_module(imp)
+    True
+    >>> is_proper_module(imp.new_module('foo'))
+    False
+
+    .. versionadded:: 1.8
+    """
+    return (inspect.ismodule(obj) and
+            obj is sys.modules.get(getattr(obj, '__name__', None)))
+
+
 #
 # Internal helpers
 #
@@ -511,7 +578,7 @@ def find_chain(obj, predicate, edge_func, max_depth=20, extra_ignore=()):
 def show_graph(objs, edge_func, swap_source_target,
                max_depth=3, extra_ignore=(), filter=None, too_many=10,
                highlight=None, filename=None, extra_info=None,
-               refcounts=False):
+               refcounts=False, shortnames=True):
     if not isinstance(objs, (list, tuple)):
         objs = [objs]
     if filename and filename.endswith('.dot'):
@@ -521,7 +588,7 @@ def show_graph(objs, edge_func, swap_source_target,
         fd, dot_filename = tempfile.mkstemp(prefix='objgraph-',
                                             suffix='.dot', text=True)
         f = os.fdopen(fd, "w")
-        if f.encoding != None:
+        if f.encoding:
             # Python 3 will wrap the file in the user's preferred encoding
             # Re-wrap it for utf-8
             import io
@@ -549,7 +616,7 @@ def show_graph(objs, edge_func, swap_source_target,
         nodes += 1
         target = queue.pop(0)
         tdepth = depth[id(target)]
-        f.write('  %s[label="%s"];\n' % (obj_node_id(target), obj_label(target, extra_info, refcounts)))
+        f.write('  %s[label="%s"];\n' % (obj_node_id(target), obj_label(target, extra_info, refcounts, shortnames)))
         h, s, v = gradient((0, 0, 1), (0, 0, .3), tdepth, max_depth)
         if inspect.ismodule(target):
             h = .3
@@ -566,7 +633,7 @@ def show_graph(objs, edge_func, swap_source_target,
             f.write('  %s_has_a_del[label="__del__",shape=doublecircle,height=0.25,color=red,fillcolor="0,.5,1",fontsize=6];\n' % (obj_node_id(target)))
         if tdepth >= max_depth:
             continue
-        if inspect.ismodule(target) and not swap_source_target:
+        if is_proper_module(target) and not swap_source_target:
             # For show_backrefs(), it makes sense to stop when reaching a
             # module because you'll end up in sys.modules and explode the
             # graph with useless clutter.  For show_refs(), it makes sense
@@ -588,7 +655,7 @@ def show_graph(objs, edge_func, swap_source_target,
                 srcnode, tgtnode = target, source
             else:
                 srcnode, tgtnode = source, target
-            elabel = edge_label(srcnode, tgtnode)
+            elabel = edge_label(srcnode, tgtnode, shortnames)
             f.write('  %s -> %s%s;\n' % (obj_node_id(srcnode), obj_node_id(tgtnode), elabel))
             if id(source) not in depth:
                 depth[id(source)] = tdepth + 1
@@ -641,8 +708,11 @@ def obj_node_id(obj):
     return ('o%d' % id(obj)).replace('-', '_')
 
 
-def obj_label(obj, extra_info=None, refcounts=False):
-    label = [type(obj).__name__]
+def obj_label(obj, extra_info=None, refcounts=False, shortnames=True):
+    if shortnames:
+        label = [type(obj).__name__]
+    else:
+        label = [long_typename(obj)]
     if refcounts:
         label[0] += ' [%d]' % (sys.getrefcount(obj) - 4)
         # Why -4?  To ignore the references coming from
@@ -663,6 +733,20 @@ def quote(s):
              .replace("\0", "\\\\0"))
 
 
+def short_typename(obj):
+    return type(obj).__name__
+
+
+def long_typename(obj):
+    objtype = type(obj)
+    name = objtype.__name__
+    module = getattr(objtype, '__module__', None)
+    if module:
+        return '%s.%s' % (module, name)
+    else:
+        return name
+
+
 def safe_repr(obj):
     try:
         return short_repr(obj)
@@ -707,7 +791,7 @@ def gradient(start_color, end_color, depth, max_depth):
     return h, s, v
 
 
-def edge_label(source, target):
+def edge_label(source, target, shortnames=True):
     if isinstance(target, dict) and target is getattr(source, '__dict__', None):
         return ' [label="__dict__",weight=10]'
     if isinstance(source, types.FrameType):
@@ -737,8 +821,11 @@ def edge_label(source, target):
                 if isinstance(k, basestring) and is_identifier(k):
                     return ' [label="%s",weight=2]' % quote(k)
                 else:
-                    return ' [label="%s"]' % quote(type(k).__name__ + "\n"
-                                                   + safe_repr(k))
+                    if shortnames:
+                        tn = type(k).__name__
+                    else:
+                        tn = long_typename(k)
+                    return ' [label="%s"]' % quote(tn + "\n" + safe_repr(k))
     return ''
 
 
